import 'package:adeptutils/adeptutils.dart';
import 'package:json_annotation/json_annotation.dart';

part 'graphql_base_error.g.dart';

@JsonSerializable()
class GraphQLBaseError extends Jsonable<GraphQLBaseError> {

  String message;
  List<Map<String, dynamic>> location;
  List<dynamic> path;
  GraphQLBaseExtension extensions;

  GraphQLBaseError({this.message, this.location, this.path, this.extensions});

  String get code => extensions?.code;
  int get statusCode => extensions?.exception?.status;

  @override
  Map<String, dynamic> toJson() => _$GraphQLBaseErrorToJson(this);
  @override
  GraphQLBaseError fromJsonMap(Map<String, dynamic> json) => GraphQLBaseError.fromJson(json);
  factory GraphQLBaseError.fromJson(Map<String, dynamic> json) => json != null ? _$GraphQLBaseErrorFromJson(json) : null;
}

@JsonSerializable()
class GraphQLBaseExtension extends Jsonable<GraphQLBaseExtension> {

  String code;
  GraphQLBaseException exception;

  GraphQLBaseExtension({this.code, this.exception});

  @override
  Map<String, dynamic> toJson() => _$GraphQLBaseExtensionToJson(this);
  @override
  GraphQLBaseExtension fromJsonMap(Map<String, dynamic> json) => GraphQLBaseExtension.fromJson(json);
  factory GraphQLBaseExtension.fromJson(Map<String, dynamic> json) => json != null ? _$GraphQLBaseExtensionFromJson(json) : null;
}

@JsonSerializable()
class GraphQLBaseException extends Jsonable<GraphQLBaseException> {

  int status;
  String message;
  List<dynamic> stacktrace;

  GraphQLBaseException({this.status, this.message, this.stacktrace});

  @override
  Map<String, dynamic> toJson() => _$GraphQLBaseExceptionToJson(this);
  @override
  GraphQLBaseException fromJsonMap(Map<String, dynamic> json) => GraphQLBaseException.fromJson(json);
  factory GraphQLBaseException.fromJson(Map<String, dynamic> json) => json != null ? _$GraphQLBaseExceptionFromJson(json) : null;
}

@JsonSerializable()
class GraphQLBaseExceptionResponse extends Jsonable<GraphQLBaseExceptionResponse> {

  int statusCode;
  String errorMessage;
  String message;
  String error;

  GraphQLBaseExceptionResponse({this.statusCode, this.errorMessage, this.message,
      this.error});

  @override
  Map<String, dynamic> toJson() => _$GraphQLBaseExceptionResponseToJson(this);
  @override
  GraphQLBaseExceptionResponse fromJsonMap(Map<String, dynamic> json) => GraphQLBaseExceptionResponse.fromJson(json);
  factory GraphQLBaseExceptionResponse.fromJson(Map<String, dynamic> json) => json != null ? _$GraphQLBaseExceptionResponseFromJson(json) : null;
}
